# -*- coding: utf8 -*-

from django.core.management.base import BaseCommand, CommandError

from gestion.inscriptions.creation_comptes import creer_compte, prochain_uid_libre
from gestion.inscriptions.models import Inscription, EtatInscr, preparer_inscription
from ldap_inp import LdapEleve
from ldap_inp.utf_tools import to_alphabetic
from portailuser.models import Ecole

import subprocess


def question(text, default=True):
    text += " [O/n] " if default else " [o/N] "
    while True:
        answer = input(text).upper()
        if answer == "":
            return default

        if answer == "O":
            return True

        if answer == "N":
            return False


class Command(BaseCommand):
    help = "Effectue les inscriptions d’une école."

    def add_arguments(self, parser):
        parser.add_argument("ecole", choices=["n7", "ensat", "a7", "enm"])

    def handle(self, *args, **options):
        try:
            ecole = Ecole.objects.get(ident=options["ecole"])
        except Exception:
            raise CommandError("École non valide.")

        noms_prenoms = set()
        for i in LdapEleve.objects.all():
            noms_prenoms.add((to_alphabetic(i.nom), to_alphabetic(i.prenom)))

        print("Inscriptions pour l’" + ecole.nom_propre + "…")

        uid = 8820

        if ecole == Ecole.enm:
            for inscr in Inscription.objects.filter(ecole=ecole, etat=1):
                uid = prochain_uid_libre(uid)
                print(f"** {inscr.prenom} {inscr.nom}… (uid = {uid})")
                prenom = to_alphabetic(inscr.prenom)
                nom = to_alphabetic(inscr.nom)
                inscr.ident_cri = f"{prenom}.{nom}"
                inscr.mail_cri = f"{prenom}.{nom}@meteo.fr"
                creer_compte(inscr, uid, "ou=people," + ecole.dn)
                inscr.etat = 3
                inscr.save()

        for inscr in Inscription.objects.filter(ecole=ecole, etat=2):
            uid = prochain_uid_libre(uid)
            print(f"** {inscr.prenom} {inscr.nom}… (uid = {uid})")

            print("Revalidation des données…")
            (code, message) = preparer_inscription(inscr, noms_prenoms, True)
            if code == EtatInscr.ERREUR:
                print(f"Données du CRI incorrectes:\n{message}\nAbandon.")
                inscr.ident_cri = ""
                inscr.mail_cri = ""
                inscr.save()
                continue

            if code == EtatInscr.AVERT:
                print(message)
                if not question("Effectuer quand même ?"):
                    continue

            creer_compte(inscr, uid, "ou=people," + ecole.dn)

            inscr.etat = 3
            inscr.save()

        inscr_invalides = Inscription.objects.filter(ecole=ecole, etat=1)

        if inscr_invalides and question("Effectuer les inscription non validées (au cas par cas) ?"):
            for inscr in inscr_invalides:
                print(f"{inscr.prenom} {inscr.nom}")
                if question("Effectuer ?", False):
                    ident_cri = input(f"Identifiant CRI : [{inscr.ident_cri}] ")

                    if ident_cri:
                        inscr.ident_cri = ident_cri

                    mail_cri = input(f"Email CRI : [{inscr.mail_cri}] ")

                    if mail_cri:
                        inscr.mail_cri = mail_cri

                    if inscr.ident_cri and inscr.mail_cri:
                        uid = prochain_uid_libre(uid)
                        print(f"En cours… (uid = {uid})")

                        creer_compte(inscr, uid, "ou=people," + ecole.dn)

                        inscr.etat = 3
                        inscr.save()

                    else:
                        print("Ignoré.")

        print("Exécution sur le serveur NFS de regler_permissions_homes.sh…")
        subprocess.call(["ssh", "root@nfs.bde.inp", "-i", "/dev/null", "/root/admin/homes.sh"])

        print("Exécution sur le serveur Web de restrictions_apache.py…")
        subprocess.call(["ssh", "root@ares.bde.inp", "/etc/cron.daily/restrictions_apache"])

        print("Terminé ! À la prochaine, camarade.")
